###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from obt import dep, host, path, command

###############################################################################

class oiio(dep.StdProvider):
  name = "oiio"
  def __init__(self): ############################################
    super().__init__(oiio.name)
    self.declareDep("cmake")
    self.declareDep("pkgconfig")
    self.declareDep("jpegturbo")
    self.declareDep("openexr")
    self.declareDep("giflib")
    self.declareDep("ffmpeg")
    BOOST = self.declareDep("boost")
    self.createBuilder(dep.CMakeBuilder)

    CMAKE_VARS = {
      "CMAKE_CXX_FLAGS": "-Wno-error=deprecated -Wno-error=ignored-attributes",
      "USE_NUKE": "OFF",
      "USE_PYTHON": "OFF",
      #"OIIO_PYTHON_VERSION": "3.8.1",
      #"pybind11_ROOT": path.stage(),
      "OIIO_BUILD_TOOLS": "ON",
      "OIIO_BUILD_TESTS": "ON",
      "JPEG_INCLUDE_DIR": path.includes(),
      "OpenCV_INCLUDE_DIR": "/dev/null", 
      "GIF_INCLUDE_DIR": path.includes(),
    }
    #if host.IsLinux:
    #  self._builder.setCmVar("CMAKE_CXX_COMPILER","g++")
    #  self._builder.setCmVar("CMAKE_C_COMPILER","gcc")



    CMAKE_VARS.update(BOOST.cmake_additional_flags())

    self._builder.setCmVars(CMAKE_VARS)
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=oiio.name,
                             repospec="OpenImageIO/oiio",
                             revision="v2.5.4.0",
                             recursive=False)

  #######################################################################
  def on_build_shell(self):
    env = {
      "CMAKE_ENVFLAGS" : self._builder.cmakeEnvAsString
    }
    print("use cmake $(echo $CMAKE_ENVFLAGS) -B . -S ..")
    return command.subshell( directory=self.build_dest,
                             prompt = "OIIO",
                             environment = env )
  #######################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()
  #######################################################################
  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libIlmImf.so").exists()



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, tarfile
from obt import dep, host, path, git, cmake, make
from obt.deco import Deco
from obt.wget import wget
from obt.command import Command

deco = Deco()

###############################################################################

class gcode_gpr(dep.Provider):

  def __init__(self): ############################################
    super().__init__("gcode_gpr")
    self.manifest = path.manifests()/"gcode_gpr"
    self.source_root = path.builds()/"gcode_gpr"
    self.build_dest = self.source_root/".build"

    self.OK = self.manifest.exists()

  def __str__(self):
    return "A simple C++ G-code parser"

  def clone(self):
    #git.Clone("https://github.com/tweakoz/gpr",self.source_root,"master")

    os.system("rm -rf %s"%self.build_dest)
    os.mkdir(self.build_dest)
    os.chdir(self.build_dest)
    cmake_ctx = cmake.context("..")
    cmake_ctx.exec()
    self.OK = (make.exec("install")==0)

  def provide(self): ##########################################################

      self.clone()
      self.manifest.touch()
      return self.OK



#!/usr/bin/env python3

import argparse, os, sys
from obt import path, pathtools, command, env, log

litex_dir = path.builds()/"litex"
litex_boards_dir = litex_dir/"litex-boards"
litex_targets_dir = litex_boards_dir/"litex_boards"/"targets"

soc_dir = path.builds()/"litex1_nx4"

parser = argparse.ArgumentParser(description="")
parser.add_argument("-l", "--load", action="store_true")
parser.add_argument('--tty', metavar="tty", default=None, help='fpga programming tty' )
options = vars(parser.parse_args())

do_load = options["load"]

tty = "/dev/ttyUSB0"

if "DEVTTY" in os.environ:
    tty = os.environ["DEVTTY"]

if options["tty"] != None:
    tty = options["tty"]

print(tty)

if do_load:
    loader = soc_dir/"software"/"bios"/"bios.bin"
    command.run([
      "djtgcfg",
      "prog",
      "-d", "Nexys4",
      "-i", "0",
      "-f", soc_dir/"gateware"/"digilent_nexys4.bit" ])
    command.run([ "litex_term",
      "--speed", "115200",
      tty])
      #"--kernel",loader])
      #"--kernel-adr","cff00000"])
else:
    cmd = [
        litex_targets_dir/"digilent_nexys4.py",
        "--output-dir", soc_dir,
        "--build",
        "--sys-clk-freq", "125e6",
        #"--cpu-type", "vexriscv_smp",
        "--cpu-variant", "linux",
        #"--with-coherent-dma",
        #"--cpu-count", "2",
        #"--with-fpu",
        #"--icache-size", "4096",
        #"--dcache-size", "4096",
        #"--with-wishbone-memory",
        #"--cpu-type", "vexriscv",
        #"--cpu-variant", "minimal",
        #"--sys-clk-freq", "75e6",
    ]
    command.run(cmd,do_log=True)



###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import platform, multiprocessing
from obt import target, sdk, command, env

class hostinfo:
  ###################################
  def __init__(self):
    self.architecture = "x86_64"
    self.os = "macos"
    self.distribution = "macos"
    revision = platform.mac_ver()[0]
    self.revision = revision
    ###################################
    self.codename = "mojave"
    sliced = int(revision[0:2])
    if(sliced==11):
      self.codename = "bigsur"
    if(sliced==12):
      self.codename = "monterey"
    if(sliced==13):
      self.codename = "ventura"
    if(sliced==14):
      self.codename = "sonoma"
    ###################################
    x86_64_macos = target.descriptor('x86_64','macos')
    aarch64_macos = target.descriptor('aarch64','macos')
    aarch64_ios = target.descriptor('aarch64','ios')
    aarch64_android = target.descriptor('aarch64','android')
    targets = set()
    targets.add(x86_64_macos)
    targets.add(aarch64_macos)
    targets.add(aarch64_ios)
    targets.add(aarch64_android)
    self.targets = targets
    self.target = x86_64_macos # host SDK
    ###################################
  ###################################
  def env_init(self):
    #x86_64_macos = sdk.descriptor('x86_64','macos')
    #x86_64_macos.env_init()
    _xcodesdkstr = command.capture([
      "xcodebuild",
      "-version",
      "-sdk","macosx"]).splitlines()
    for l in _xcodesdkstr:
      x = l.split(": ")
      if x[0]=="Path":
        env.set("OBT_MACOS_SDK_DIR",x[1])
      if x[0]=="PlatformVersion":
        env.set("OBT_MACOS_PLATFORM_VERSION",x[1])
    print("MACOS-X86_64 Host Activated...")
  ###################################
  @property
  def num_cores(self):
    return multiprocessing.cpu_count()
  ###################################
  @property
  def identifier(self):
    return "x86_64-linux"
    



###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, tarfile
from obt import dep, host, path, git, cmake, make
from obt.deco import Deco
from obt.wget import wget
from obt.command import Command

deco = Deco()

###############################################################################

class simavr(dep.Provider):

  def __init__(self): ############################################
    super().__init__("simavr")
    self.manifest = path.manifests()/"simavr"
    self.source_root = path.builds()/"simavr"
    self._archlist = ["x86_64"]

  def provide(self): ##########################################################

    git.Clone("https://github.com/tweakoz/simavr",self.source_root,"master")
    os.chdir(self.source_root)
    os.environ["INSTALL_PREFIX"] = str(path.prefix())
    make.exec("install")
    self.manifest.touch()
    self.OK = self.manifest.exists()
    return self.OK



#!/usr/bin/env python3

import torch
import torchvision
import torchvision.transforms as transforms

# PyTorch TensorBoard support
from torch.utils.tensorboard import SummaryWriter
from datetime import datetime


transform = transforms.Compose(
    [transforms.ToTensor(),
    transforms.Normalize((0.5,), (0.5,))])

# Create datasets for training & validation, download if necessary
training_set = torchvision.datasets.FashionMNIST('./data', train=True, transform=transform, download=True)
validation_set = torchvision.datasets.FashionMNIST('./data', train=False, transform=transform, download=True)

# Create data loaders for our datasets; shuffle for training, not for validation
training_loader = torch.utils.data.DataLoader(training_set, batch_size=4, shuffle=True, num_workers=2)
validation_loader = torch.utils.data.DataLoader(validation_set, batch_size=4, shuffle=False, num_workers=2)

# Class labels
classes = ('T-shirt/top', 'Trouser', 'Pullover', 'Dress', 'Coat',
        'Sandal', 'Shirt', 'Sneaker', 'Bag', 'Ankle Boot')

# Report split sizes
print('Training set has {} instances'.format(len(training_set)))
print('Validation set has {} instances'.format(len(validation_set)))


###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import platform, os, pathlib,sys
from enum import Enum
import obt.path 
import obt.env 

###############################################################################

file_path = os.path.realpath(__file__)
this_dir = pathlib.Path(os.path.dirname(file_path))

###############################################################################

def current():
  return os.environ.get("OBT_SUBSPACE")

###############################################################################

def module_class(module_path,subname):
  import obt.module
  the_module = obt.module.instance("sub_"+subname,module_path)
  if the_module != None:
    return the_module.subspaceinfo
  return None 

###############################################################################

def subspace_dirs():
  subspace_dirs_list = list()
  module_dirs_list = os.environ["OBT_MODULES_PATH"].split(":")
  for module_dir in module_dirs_list:
    subspace_path = obt.path.Path(module_dir)/"subspace"
    if subspace_path.exists():
      subspace_dirs_list += [subspace_path]
  return subspace_dirs_list

###############################################################################

def descriptor(subname):
  subspace_dirs_list = subspace_dirs()
  for subspace_dir in subspace_dirs_list:
    try_path = subspace_dir/("%s.py"%subname)
    if try_path.exists():
      return module_class(try_path,subname)()


###############################################################################

def requires(subname,build_opts=[]):
  sub = descriptor(subname)
  if not sub._manifest_path.exists():
    sub.build(build_opts) # todo skip if manifest present
    sub._manifest_path.touch()
  return sub 

###############################################################################

def enumerate():
  #######################################
  class EnumItem:
    def __init__(self,name,fullpath,module):
      self._name = name 
      self._fullpath = fullpath
      self._module = module
  #######################################
  module_dict = dict()
  module_dirs_list = os.environ["OBT_MODULES_PATH"].split(":")
  for module_dir in module_dirs_list:
    #print(dep_repo)
    subspace_path = obt.path.Path(module_dir)/"subspace"
    if subspace_path.exists():
      path_list = os.listdir(subspace_path)
      for item in path_list:
        module_test_path = subspace_path/item
        has_py_ext = (str(module_test_path).find(".py")>0)
        if module_test_path.exists() and has_py_ext:
          mclass = module_class(module_test_path,item)
          module = mclass()
          if module!=None:
            e = EnumItem(item,module_test_path,module)
            module_dict[item] = e
  return module_dict

def findWithMethod(named):
  e = enumerate()
  rval = {}
  for k in e.keys():
    item = e[k]
    module = item._module
    if module and hasattr(module,named):
      rval[k] = item
  return rval



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, inspect, tarfile
from pathlib import Path
import importlib.util
import obt.path, obt.host
from obt.command import Command, run
from obt.deco import Deco
from obt.wget import wget
from obt import pathtools, cmake, make, path, git, host, _globals, log
from obt._dep_impl import require
from collections.abc import Callable

deco = Deco()

###############################################################################
# BaseBuilder : builder basic interface
###############################################################################

class BaseBuilder(object):
  def __init__(self,name):
    super().__init__()
    self._name = name
    self._deps = []
    self._debug = False
    self._onPostBuild = None
    self._onPostInstall = None
  def requires(self,deplist):
    """declare that this dep module depends on others"""
    self._deps += deplist
  def build(self,srcdir,blddir,wrkdir,incremental=False):
    """execute build phase"""
    require(self._deps)
  def install(self,blddir):
    """execute install phase"""
    return True

###############################################################################
# NopBuilder : do nothing builder (but still installs child dependencies)
###############################################################################

class NopBuilder(BaseBuilder):
  """Do nothing builder (but still installs child dependencies)"""
  def __init__(self,name):
    super().__init__(name)
  def build(self,srcdir,blddir,wrkdir,incremental=False):
    return True
###############################################################################
# BinInstaller : install binaries from downloaded package
###############################################################################

class BinInstaller(BaseBuilder):
  """Install binaries from downloaded package"""
  ###########################
  class InstallerItem:
    def __init__(self,src,dst,flags):
      self._src = src
      self._dst = dst
      self._flags = flags
  ###########################
  def __init__(self,name):
    super().__init__(name)
    self._items = []
    self._OK = True
  ###########################################
  """declare an install item given a source-path and dest-path"""
  def install_item(self,
                   source=None,
                   destination=None,
                   flags=None):
    self._items += [BinInstaller.InstallerItem(source,destination,flags)]
  ###########################################
  def build(self,srcdir,blddir,wrkdir,incremental=False):
    ok2build = require(self._deps)
    if not ok2build:
      return False
    for item in self._items:
      exists = item._src.exists()
      #print(item,item._src,exists)
      if False==exists:
        self._OK = False
        return False
    return True
  ###########################################
  def install(self,blddir):
    for item in self._items:
      cmd = [
        "cp", item._src, item._dst
      ]
      Command(cmd).exec()
      if item._flags != None:
        cmd = [
          "chmod",item._flags, item._dst
        ]
        Command(cmd).exec()
    return self._OK

###############################################################################

class CMakeBuilder(BaseBuilder):
  ###########################################
  def __init__(self,
               name,
               static_libs=False,
               macos_defaults=True,
               install_prefix=None):
    super().__init__(name)
    self._minimal = False 
    self._install_prefix = install_prefix
    ##################################
    # ensure environment cmake present
    ##################################
    self._cmakeenv = {
      "CMAKE_BUILD_TYPE": "Release",
    }
    self._osenv = {
    }

    if not static_libs:
      self._cmakeenv["BUILD_SHARED_LIBS"]="ON"

    ##################################
    # default OSX stuff
    ##################################
    if obt.host.IsOsx and macos_defaults:
      sysroot_cmd = Command(["xcrun","--show-sdk-path"],do_log=False)
      sysroot = sysroot_cmd.capture().replace("\n","")

      if obt.host.IsAARCH64:
        self._cmakeenv.update({"CMAKE_HOST_SYSTEM_PROCESSOR":"arm64"})
      else:
        self._cmakeenv.update({"CMAKE_HOST_SYSTEM_PROCESSOR":"x86_64"})

      self._cmakeenv.update({
        "CMAKE_OSX_DEPLOYMENT_TARGET:STRING":"11",
        "CMAKE_OSX_SYSROOT:STRING":sysroot,
        "CMAKE_MACOSX_RPATH": "1",
        "CMAKE_INSTALL_RPATH": path.libs(),
        "CMAKE_SKIP_INSTALL_RPATH:BOOL":"NO",
        "CMAKE_SKIP_RPATH:BOOL":"NO",
        "CMAKE_INSTALL_NAME_DIR": "@executable_path/../lib"
      })

    ##################################
    self._parallelism = 0.0 if _globals.tryBoolOption("serial") else 1.0
    ##################################
    # implicit dependencies
    ##################################
    if name!="cmake":
      self._deps += ["cmake"]
  ###############################################
  @property 
  def install_prefix(self):
    return path.prefix() if (self._install_prefix==None) else self._install_prefix
  ###########################################
  def requires(self,deplist):
    self._deps += deplist
  ###########################################
  def setCmVar(self,key,value):
    self._cmakeenv[key] = value
  ###########################################
  def setCmVars(self,othdict):
    for k in othdict:
      self._cmakeenv[k] = othdict[k]
  ###########################################
  @property 
  def cmakeEnvAsString(self):
    return " ".join(self.cmakeEnvAsStringList)
  ###########################################
  @property 
  def cmakeEnvAsStringList(self):
    args = []
    for k in self._cmakeenv:
      v = self._cmakeenv[k]
      args += ["-D%s=%s"%(k,v)]
    return args
  ###########################################
  def build(self,srcdir,blddir,wrkdir,incremental=False):
    print("srcdir<%s>"%srcdir)
    print("blddir<%s>"%blddir)
    print("wrkdir<%s>"%wrkdir)

    ok2build = require(self._deps)
    if not ok2build:
      return False
    if incremental:
      pathtools.mkdir(blddir,clean=False)
      pathtools.chdir(wrkdir)
      cmake_ctx = cmake.context(root=srcdir,
                                env=self._cmakeenv,
                                osenv=self._osenv,
                                builddir=blddir,
                                working_dir=wrkdir,
                                install_prefix=self.install_prefix)
      ok2build = cmake_ctx.exec()==0
    else:
      pathtools.mkdir(blddir,clean=True)
      pathtools.chdir(wrkdir)
      cmake_ctx = cmake.context(root=srcdir,
                                env=self._cmakeenv,
                                osenv=self._osenv)
      ok2build = cmake_ctx.exec()==0

    if ok2build:
      OK = (make.exec(parallelism=self._parallelism)==0)
      if OK and self._onPostBuild!=None:
        self._onPostBuild()
      return OK
    return False
  ###########################################
  def install(self,blddir):
    pathtools.chdir(blddir)
    OK = (make.exec("install",parallelism=0.0)==0)
    if OK and self._onPostInstall!=None:
       self._onPostInstall()
    return OK

  ###########################################

###############################################################################

class AutoConfBuilder(BaseBuilder):
  ###########################################
  def __init__(self,name):
    super().__init__(name)
    ##################################
    self._needaclocal = False
    self._needsautogendotsh = False
    self._parallelism=1.0
    self._options = list()
    self._envvars = dict()
    if _globals.tryBoolOption("serial"):
      self._parallelism=0.0
  ###########################################
  def requires(self,deplist):
    self._deps += deplist
  ###########################################
  def setEnvVar(self,key,value):
    self._envvars[key] = value
  ###########################################
  def setConfVar(self,key,value):
    self._confvar[key] = value
  ###########################################
  def setOption(self,opt):
    self._options += [opt]
  ###########################################
  def setConfVars(self,othdict):
    for k in othdict:
      self._confvar[k] = othdict[k]
  ###########################################
  def build(self,srcdir,blddir,wrkdir,incremental=False):
    ok2build = require(self._deps)
    if not ok2build:
      return False
    retc = 0
    pathtools.mkdir(blddir)
    if incremental:
      os.chdir(blddir)
    else:

      if self._needaclocal:
        pathtools.chdir(srcdir)
        retc = Command(["aclocal"]).exec()
        if retc!=0:
          print(deco.red("Error running aclocal<%d>"%retc))
          return False
      if self._needsautogendotsh:
        pathtools.chdir(srcdir)
        retc = Command(["./autogen.sh"]).exec()
        if retc == 0:
           make.exec("distclean")
        else:
          print(deco.red("Error running autogen.sh<%d>"%retc))
          return False

      pathtools.mkdir(blddir,clean=True)
      pathtools.chdir(blddir)

      retc = Command([srcdir/"configure",
                      '--prefix=%s'%path.prefix()
                     ]+self._options,
                     environment=self._envvars
                     ).exec()
      if retc==0:
        make.exec("all")
        retc = make.exec("install",parallelism=0.0)
    print("retc<%d>"%int(retc))
    return retc==0
  ###########################################
  def install(self,blddir):
    pathtools.chdir(blddir)
    return (make.exec("install",parallelism=0.0)==0)
  ###########################################

###############################################################################

class CustomStep:
  def __init__(self,name,funktor):
    super().__init__()
    self._name = name
    self._funktor = funktor 

###############################################################################

class CustomBuilder(BaseBuilder):
  ###########################################
  def __init__(self,name):
    super().__init__(name)
    ##################################
    self._envvars = dict()
    self._parallelism=1.0
    self._cleanbuildcommands = list()
    self._incrbuildcommands = list()
    self._installcommands = list()
    self._builddir = path.builds()/name

    if _globals.tryBoolOption("serial"):
      self._parallelism=0.0
  ###########################################
  def requires(self,deplist):
    self._deps += deplist
  ###########################################
  def setEnvVar(self,key,value):
    self._envvars[key] = value
  ###########################################
  def setEnvVars(self,othdict):
    for k in othdict:
      self._envvars[k] = othdict[k]
  ###########################################
  def _run_commands(self,cmdlist):
    if len(cmdlist)>0:
      for cmd in cmdlist:
        if isinstance(cmd,Command):
          retc = cmd.exec()
          if retc!=0:
            return False
        elif isinstance(cmd,CustomStep):
          retc = cmd._funktor()
          if retc==False:
            return False
        elif isinstance(cmd,Callable):
          if cmd()==False:
            return False
    return True
  ###########################################
  def build(self,srcdir,blddir,wrkdir,incremental=False):
    print("srcdir<%s>"%srcdir)
    print("blddir<%s>"%blddir)
    print("incremental<%s>"%incremental)
    print("deps<%s>"%self._deps)
    ok2build = require(self._deps)
    if not ok2build:
      return False
    ###################################
    if incremental:
    ###################################
      pathtools.mkdir(self._builddir,clean=False,parents=True)
      pathtools.chdir(self._builddir)
      return self._run_commands(self._incrbuildcommands)
    ###################################
    else: # clean build
    ###################################
      pathtools.mkdir(self._builddir,clean=False)
      pathtools.chdir(self._builddir)
      return self._run_commands(self._cleanbuildcommands)
  ###########################################
  def install(self,blddir):
    pathtools.chdir(self._builddir)
    return self._run_commands(self._installcommands)
  ###########################################
